/*
	Crossing
	Copyright (C) 2007 Michal Vaner (michal.vaner@kdemail.net)

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	he Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


#include "generator.h"
#include "point.h"
#include "load.h"
#include "vehicle.h"

Generator::Generator(const QDomElement &node, Point *point) :
	QObject(point->indexer()),//Will take care of destruction
	mPoint(point)
{
	if(!(vehicType = vehicTypes().value(node.attribute("type"))))
		throw XMLLoadDataE(tr("Unknown vehicle type %1").arg(node.attribute("type")));
	bool ok;
	mAmount = node.attribute("amount").toInt(&ok);
	if(!ok)
		throw XMLLoadDataE(tr("Vehicle amount not specified"));
	connect(point->indexer(), SIGNAL(tick()), SLOT(tick()));
}

void Generator::tick() {
	if((random() % (mPoint->indexer()->traffic() * 10)) < mAmount) {
		if(!mPoint->isUnlocked()) {
			return;//No luck, nowhere to appear
		}
		Vehicle *vehicle = new Vehicle(vehicType, mPoint);
		mPoint->immediateLock(vehicle);
		mPoint->indexer()->insertIndexable(vehicle);
	}
}
