/*
	Crossing
	Copyright (C) 2007 Michal Vaner (michal.vaner@kdemail.net)

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	he Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef LOAD_H
#define LOAD_H

#include <QString>
#include <exception>

//Some exceptions

class LoadUnknownE : public std::exception {
		virtual void x();//This is here to make sure C++ recognizes the exception classes
	public:
		virtual ~LoadUnknownE() throw();
};

class XMLLoadFormatE : public LoadUnknownE {
		virtual void x();
	public:
		virtual ~XMLLoadFormatE() throw();
};

class XMLLoadForeignE : public LoadUnknownE {
		virtual void x();
	public:
		virtual ~XMLLoadForeignE() throw();
};

class XMLLoadDataE : public LoadUnknownE {
		virtual void x();
	public:
		//Additional info about the error
		const QString text;
		virtual ~XMLLoadDataE() throw();
		XMLLoadDataE(const QString _text) : text(_text) {};
};

class XMLLoadFileNotFoundE : public LoadUnknownE {
		virtual void x();
	public:
		virtual ~XMLLoadFileNotFoundE() throw();
};

class ImageNotFoundE : public LoadUnknownE {
		virtual void x();
	public:
		const QString filename;
		virtual ~ImageNotFoundE() throw();
		ImageNotFoundE(const QString &fn) : filename(fn) {};
};
//enough of exceptions

class Crossget;
class Indexer;

//Loads and initializes the crossing
Indexer *loadXmlFile(const QString &fn, Crossget *crossget);

#endif//LOAD_H
