/*
	Crossing
	Copyright (C) 2007 Michal Vaner (michal.vaner@kdemail.net)

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	he Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


#ifndef POINT_H
#define POINT_H

#include "indexer.h"
#include "behav.h"
#include "generator.h"

#include <QString>
#include <QDomElement>
#include <QHash>

class Vehicle;

class Point : public QObject, public Indexable {
	Q_OBJECT;
	private:
		int mX, mY;
		QString mId;
		const QDomElement mNode;
		QHash<QString, Line *> lines;//Lines going out by their id
		QHash<QString, Line *> inLines;//Lines going in by their id
		std::auto_ptr<RootCond> behavior;
		std::auto_ptr<Generator> generator;
		bool lockDecided;
		::Vehicle *lockedBy;
		struct LockRequest {
			::Vehicle *by;
			int priority;
			bool near;
		};
		bool better(const LockRequest &_1, const LockRequest &_2) const;
		QList<LockRequest> lockReqs;
		void decideLock();
	public:
		void processVehicle(::Vehicle *vehicle);
		QString id() const { return mId; }
		Point(const QDomElement &pointNode, Indexer &indexer);
		void pushLine(Line *ln);//This and the one below is for constructing the structure. Always call this and the one below on the dest point together!
		void pushBackLine(Line *ln);
		int x() const { return mX; }
		int y() const { return mY; }
		void constructStructure();//Load the structure from the node. Call only once and before the DOM document is deleted
		Line *inLine(const QString &id) { return inLines.value(id); }
		Line *line(const QString &id) { return lines.value(id); }
		QList<Line *> allOutLines() { return lines.values(); }
		VehicState decide(const ::Vehicle &vehicle) const;
	private slots:
		void clearLock(QObject *deadVehicle);
		void newLockRound();
		void clearLockReqs();
		void reinitLocks();
	public:
		void reserveLock(::Vehicle *vehicle, int priority, bool near);
		bool hasLock(::Vehicle *vehicle);
		void giveUpLock(::Vehicle *vehicle);
		bool isUnlocked();
		void immediateLock(::Vehicle *vehicle);
};

#endif//POINT_H
