/*
	Crossing
	Copyright (C) 2007 Michal Vaner (michal.vaner@kdemail.net)

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	he Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


#ifndef VEHICLE_H
#define VEHICLE_H

#include "indexer.h"
#include "point.h"
#include "behav.h"

#include <QObject>
#include <QString>
#include <QVector>
#include <QHash>
#include <QPixmap>

class Line;

class VehicType {
	public:
		class ImageSet {
			private:
				QVector<QPixmap> mImgs;
				int mImgInt;
			public:
				int imgInt() const { return mImgInt; }
				const QVector<QPixmap> &imgs() const { return mImgs; }
				ImageSet(const QString &ln, const  QString &prefix);
		};
	private:
		QVector<const ImageSet *> mImageSets;
		QString mType;
	public:
		const QString &type() const { return mType; }
		const QVector<const ImageSet *> sets() const { return mImageSets; }
		VehicType(const QString &fn);
};

const QHash<QString, const VehicType *> &vehicTypes();

void loadVehicTypes();

class Vehicle : public QObject, public Indexable {
	Q_OBJECT;
	private:
		const VehicType::ImageSet *imageSet;
		int imgPos, imgTime;
		Point *mPoint;//What point did I wisit last?
		Line *mLine;//What line do I go by?
		int srcSpeed;//The speed of the point I was in
		void moveToPoint();
		double actPos;
		bool waiting;
		bool tryLock(Point *point);
		void unlockIfLocked(Point *point);
	public:
		Vehicle *waitForVehicle;
		const VehicType *vtype;
		Vehicle(const VehicType *type, Point *point);
		const QPixmap &img() const { return imageSet->imgs()[imgPos]; }
		double rotation() const;
		int x() const;
		int y() const;
		const Line *actLine() const { return mLine; }
	private slots:
		void timeTick(int timeChange);
		void canGo(int timeChange);
	private:
		VehicState nextPoint;//What will I do in the next point and how do I get there?
};

#endif
